      Subroutine RSPC77(name,R,npt,xsp,xep,xt,yt,res,memo,ier)
C     Version 1
C     A FORTRAN 77 subroutine to read the .SPC files used in Galactic 
C     Industries SPECTRACALC
C     Version 1 written by John Bertie, April 1997.
c     Modified to version 2 by J. Bertie,November 14, 1997 to implement
C     Y.Apelblat's changes to correctly read and write negative values
C     of ISCALE

C***********************************************************
C     The .SPC file structure is:
C  Byte 1,      Version Number (optional, and usually not used)
C  Byte 2,      must be set to 4D hex, i.e. 77 in base 10
C  Bytes 3&4,   integer ISCALE, the exponent of 2 for all Y values
C  Bytes 5-8,   Real RNPT, the number of Y values
C  Bytes 9-12,  Real XSP, wavenumber of the first Y value
C  Bytes 13-16, Real XEP, wavenumber of the last Y value    
C  Byte 17,     Type of X value, 1 for wavenumber
C  Byte 18,     Type of Y value, 0 for arbitrary
C  Bytes 19-20   Year, for a new file, that the spectrum was collected
C  Bytes 21-24   Month (21), day (22), hour (23), minute (24).  
C  Bytes 25-32   RESO, Resolution of data as ASCII text. Ignores bytes 
C                      after first zero
C  Bytes 33-64   Reserved for 8 REAL values
C  Bytes 65-256  The 192 char MEMO descriptive of the spectrum.
C                 THE MEMO MUST END WITH a zero byte
C  Bytes 257 to end of file   The Y values as a signed 32-bit word reversed 
C             fractions.  Before use, the first two bytes must be 
C             interchanged with the last two bytes.     
C
C     Bytes 1 to 4 and 17 to 20 require tricks when integer *1 is not
C     available.They must be read as 4-byte groups.  FORTRAN reverses
C     the BYTE order when it reads bytes as integer words.  
C     Thus, the 4 byte groups are read as the CHARACTER*1 array BYT, 
C     with the elements read in reverse order. The 4-byte integer WD 
C     is equivalenced to BYT(1) so that BYT(1) through BYT(4) occupy
C     the same memmory locations as WD.  
C     The WORD WD can then be separated into its component bytes by 
C     the subroutines SEP2 and SEP4.  
C     The Y data is treated in an analogous way, modified to suit the
C     signed 32-bit word reversed fractions format used by Spectracalc.
C***********************************************************
C    PASSED ARGUMENTS , in the sequence:        NAME    TYPE    USAGE
C       ----    ----    ------------------------------------
C       name   CHAR   THE FILENAME TO READ ON DISK
C       R      ARRAY  REAL ARRAY WITH Y DATA
C       npt    INT    NUMBER OF POINTS OF Y DATA
C       xsp    REAL   FLOATING POINT VALUE- FIRST X
C       xep    REAL   FLOATING POINT VALUE- LAST X
C       xt     INT    X DATA TYPE (SEE SC MANUAL)
C       yt     INT    Y DATA TYPE (SEE SC MANUAL)
C       res    REAL   NOMINAL DATA RESOLUTION (from RESO)
C       memo   CHAR   ARRAY WITH 192 BYTE COMMENT
C       IER    INT       ERROR CODE  0=NO ERROR
C***********************************************************
C
C       *** INITIALIZE 
      CHARACTER*1 BYT(4)
      CHARACTER*8 reso
      CHARACTER*12 name
      CHARACTER*192 memo
      INTEGER npt,iscale,maxint,xt,yt,ier,verd4,iyr,itop,WD,
     X IMON,IDAY,IHR,IMIN
      REAL R(1),reserv(8),rnpt,factor,xsp,xep,res
      EQUIVALENCE (WD,BYT)
      
C   Maxint is the largest integer in a 32-bit word, 2**31 -1
      PARAMETER (maxint=2147483647)

      IER = 0
C     OPEN THE FILE
      CALL GETCHN(ICHAN,IER)
	OPEN(UNIT=ICHAN,FILE=NAME,FORM='BINARY',
     XSTATUS='UNKNOWN',ERR=9999)
      
C    READ THE FIRST 4-BYTE WORD & SEPARATE ITS TOP 2 BYTES FROM THE BOTTOM 2 
C    TO GIVE VERD4 (Galactic VER + #4D; top 2) AND ISCALE (bottom 2). 
C    BYT(1) and BYT(2) ARE INTERCHANGED BECAUSE ISCALE IS A 2-BYTE WORD
C    Also read the next three 4-byte words to get rnpt, xsp & xep directly.   
      READ(ICHAN)BYT(4),BYT(3),BYT(1),BYT(2),rnpt,xsp,xep
      npt=int(rnpt)
      CALL sep2(WD,verd4,iscale)
C    READ THE 5TH WORD, BYTES 17-24.
C    BYT(1) and BYT(2) ARE INTERCHANGED BECAUSE IYR IS A 2-BYTE WORD  
C    SEPARATE IT INTO ITS BYTE 1 (XT), 2 (YT), AND BYTES 3&4 (iyr).
C    itop contains zeros in its 1st & 2nd. bytes, xt in its 3rd. 
C    and yt in its 4th.
      READ(ICHAN)BYT(4),BYT(3),BYT(1),BYT(2)
      CALL sep2(WD,itop,iyr)
      CALL sep4(itop,jk1,jk2,xt,yt)   
C     READ WORD 6. IT CONTAINS THE MONTH,DAY,HOUR AND MINUTE.  
C     WORD 6 IS USUALLY NOT USED.
      READ(ICHAN)BYT(4),BYT(3),BYT(2),BYT(1)
      CALL sep4(WD,IMON,IDAY,IHR,IMIN) 
C    READ RESO (8 bytes), reserv (32 bytes) and MEMO (192 bytes)
      READ(ICHAN)reso,reserv,memo
      
C    CONVERT RESO TO RES
      READ(reso,'(F4.1,I1)',err=1)res       

C    READ THE Y DATA

C     The IF statement compensates for negative iscale values which are stored
C     as complement integers (i.e. -2 is stored as FF FE or 65534)

    1 if(iscale.gt.32767) iscale=iscale-65536

      factor = maxint/2.0**(iscale-1)
      DO 2 I=1,npt
      READ(ICHAN)BYT(3),BYT(4),BYT(1),BYT(2)
      R(I)=REAL(WD)/factor
    2 CONTINUE            
C       *** FINISH UP AND RETURN NORMAL
      CALL PUTCHN(ICHAN,IER)
      RETURN
C       ERROR HANDLING AND ABNORMAL RETURNS
 9999 STOP 'ERROR OPENING SPECTRA-CALC FILE IN RSPC77'
      RETURN
      END
      
      
      SUBROUTINE WSPC77(name,R,npt,xsp,xep,xt,yt,res,memo,ier)
C     Version 2
C     A FORTRAN 77 subroutine to write the .SPC files used in Galactic 
C     Industries SPECTRACALC
C     Version 1 written by John Bertie, April 1997.
C     Modified to Version 2 by Yoram Apelblat, May 12, 1997 to remove 
C     the memo input from the subroutine WSPC77
c     Modified to version 3 by J. Bertie,November 14, 1997 to implement
C     Y.Apelblat's changes to correctly read and write negative values
C     of ISCALE

C***********************************************************
C     The .SPC file structure is:
C  Byte 1,      Version Number (optional, and usually not used)
C  Byte 2,      must be set to 4D hex,  i.e.  #4D
C  Bytes 3&4,   integer ISCALE, the exponent of 2 for all Y values
C  Bytes 5-8,   Real RNPT, the number of Y values
C  Bytes 9-12,  Real XSP, wavenumber of the first Y value
C  Bytes 13-16, Real XEP, wavenumber of the last Y value    
C  Byte 17,     Type of X value, 1 for wavenumber
C  Byte 18,     Type of Y value, 0 for arbitrary
C  Bytes 19-20   Year, for a new file, that the spectrum was collected
C  Bytes 21-24   Month (21), day (22), hour (23), minute (24).  
C  Bytes 25-32   RESO, Resolution of data as ASCII text. Ignores bytes 
C                      after first zero
C  Bytes 33-64   Reserved for 8 REAL values
C  Bytes 65-256  The 192 char MEMO descriptive of the spectrum.
C                 THE MEMO MUST END WITH a zero byte
C  Bytes 257 to end of file   The Y values as a signed 32-bit word reversed 
C             fractions.  Before use, the first two bytes must be 
C             interchanged with the last two bytes. 

C     Bytes 1 to 4 and 17 to 20 require tricks when integer *1 is not
C     available.They must be read as 4-byte groups.  FORTRAN reverses
C     the BYTE order when it reads bytes as integer words.  
C     Thus, the 4 byte groups are read as the CHARACTER*1 array BYT, 
C     with the elements read in reverse order. The 4-byte integer WD 
C     is equivalenced to BYT(1) so that BYT(1) through BYT(4) occupy
C     the same memory locations as WD.    
C     The output bytes can be built into the 4-byte integer
C     WD by Comb2 and COMB4 and then written in the order 
C     BYT(4) through BYT(1).
C     The Y data is treated in an analogous way, modified to suit the
C     signed 32-bit word reversed fractions format used by Spectracalc.
C***********************************************************
C       PASSED ARGUMENTS , in the sequence:     NAME    TYPE    USAGE
C       ----    ----    ------------------------------------
C       name   CHAR   THE FILENAME TO READ ON DISK
C       R      ARRAY  REAL ARRAY WITH Y DATA
C       npt    INT    NUMBER OF POINTS OF Y DATA
C       xsp    REAL   FLOATING POINT VALUE- FIRST X
C       xep    REAL   FLOATING POINT VALUE- LAST X
C       xt     INT    X DATA TYPE (SEE SC MANUAL)
C       yt     INT    Y DATA TYPE (SEE SC MANUAL)
C       res    REAL   NOMINAL DATA RESOLUTION (from RESO)
C       memo   CHAR   ARRAY WITH 192 BYTE COMMENT
C       IER    INT       ERROR CODE  0=NO ERROR
C***********************************************************
C
C       *** INITIALIZE
      CHARACTER*1 BYT(4)
      CHARACTER*8 reso
      CHARACTER*12 name
      CHARACTER*192 memo
      INTEGER ver,d4,npt,iscale,maxint,xt,yt,ier,WD,isp,b3,b4,b12
      REAL    R(*),reserv(8),factor,xsp,xep,res,rmax,rmin
      EQUIVALENCE (WD,BYT)  
C   Maxint is the largest integer in a 32-bit word, 2**31 -1
      PARAMETER (maxint=2147483647)

      ier=0
C     OPEN THE FILE
      CALL GETCHN(ICHAN,IER)
	OPEN(UNIT=ICHAN,FILE=NAME,FORM='BINARY',
     XSTATUS='UNKNOWN',ERR=9999)

C    DETERMINE SCALING EXPONENT ISCALE
      isp=1
      CALL ahilo(R,isp,npt,rmax,rmin)
C    IF RMAX>0.5      
      rmax=ABS(rmax)
      IF(abs(rmin).GT.rmax) rmax=abs(rmin)
      IF(rmax.ge.0.5)THEN 
      
	 iscale=0
    1    IF((rmax/2.00**iscale).LT.0.5) GO TO 10
	 iscale=iscale+1
	 GO TO 1

	ELSEIF( RMAX .GT. 0. )  THEN

	   iscale=0
    2    IF( rmax/(2.0**(iscale-1.0)) .GT. 0.5 ) GO TO 10
	   iscale=iscale-1
	   GO TO 2

	ELSE
	STOP 'NO VALUE GT ZERO'
	ENDIF

   10 CONTINUE  

C     CREATE THE INTEGER WORD THAT CONTAINS VER AND D4 IN ITS BYTES 1&2
C     The IF statement compensates d4 for negative iscale values in order
C     to get the correct 4D hex in WD 
      ver=0
      d4=77
      if(iscale.lt.0) d4=78
      B3=0
      B4=0
      CALL comb4(B12,ver,d4,b3,b4)

C    COMBINE B12 (BYTES 1&2) AND ISCALE (BYTES 3&4) INTO ONE 32-BIT WORD
      WD = B12 + iscale  
C    WRITE THE FIRST 4 WORDS TO DISK. BYTs 1 & 2 ARE SWITCHED AS IN RSPC
      WRITE(ICHAN)BYT(4),BYT(3),BYT(1),BYT(2)
      WRITE(ICHAN)REAL(npt),xsp,xep
C    COMBINE XT AND YT WITH ZERO FOR THE YEAR to form THE 5th WORD
      CALL comb4(WD,xt,yt,b3,b4)
      WRITE(ICHAN)BYT(4),BYT(3),BYT(1),BYT(2)
C    SET THE MONTH,DAY,HOUR & MINUTE TO ZERO      
      WD=0
      WRITE(ICHAN)BYT(4),BYT(3),BYT(2),BYT(1)
C    NOTE, I HAVE ARBITRARILY SET THE YEAR,MONTH,DAY,HOUR & MINUTE TO 
c    ZERO.  UNLESS THE DATE & TIME THE ORIGINAL SPECTRUM WAS RECORDED 
c    IS SAVED, THE DATE & TIME STORED HERE WOULD BE WHEN THE FILE WAS
c    CONVERTED TO .SPC FORMAT.  THIS IS USUALLY NOT SIGNIFICANT.
      DO 3 lz=1,8
      reserv(lz)=0
    3 CONTINUE
      WRITE(reso,'(F4.1,I1)')res,0
      WRITE(ICHAN) reso,reserv,memo
C    SCALE AND STORE THE Y VALUES
      factor=maxint/2.0**(iscale-1)
      DO 4 I=1,npt
      WD=R(I)*factor
      WRITE(ICHAN)BYT(3),BYT(4),BYT(1),BYT(2)
    4 CONTINUE
C       *** FINISH UP AND RETURN NORMAL
      CALL PUTCHN(ICHAN,IER)
      RETURN
C       ERROR HANDLING AND ABNORMAL RETURNS
 9999 STOP 'ERROR OPENING SPECTRA-CALC FILE IN WSPC77'
      RETURN
      END
      
      
      SUBROUTINE sep4(v32,v1,v2,v3,v4)
C    WRITTEN BY JOHN BERTIE, APRIL 1997
C    TO SEPARATE A 32-BIT INTEGER WORD, V32, INTO 4 32-BIT INTEGER 
C    WORDS THAT EACH HAS THE VALUE OF ONE OF ITS BYTES, WHICH ARE 
C    RETURNED AS V1,V2, V3 AND V4, WHERE V1 IS THE HIGHEST BYTE AND 
C    V4 IS THE LOWEST.
      
      INTEGER v32,v24,v16,v1,v2,v3,v4,f3,f2,f1 
C    f3 is 256**3, f2 is 256**2 and f1 is 256     
      PARAMETER (f3=16777216,f2=65536,f1=256)

      v1=v32/f3
      v24=v32-v1*f3
      v2=v24/f2
      v16=v24-v2*f2
      v3=v16/f1
      v4=v16-v3*f1
      RETURN
      END
      
      
      SUBROUTINE comb4(v32,v1,v2,v3,v4)
C    WRITTEN BY JOHN BERTIE, APRIL 1997
C    TO COMBINE 4 32-BIT INTEGERS, EACH LESS THAN 256, INTO ONE 32-BIT
C    INTEGER IN THE SEQUENCE V1(HIGHEST BYTE),V2,V3,V4 (LOWEST BYTE)
      
      INTEGER v32,v1,v2,v3,v4,f1,f2,f3
C    f3 is 256**3, f2 is 256**2 and f1 is 256
      PARAMETER (f3=16777216,f2=65536,f1=256)     

      v32=v1*f3+v2*f2+v3*f1+v4
      RETURN
      END
      
      
      
      SUBROUTINE sep2(v1234,v12,v34)
C    WRITTEN BY JOHN BERTIE, APRIL 1997
C    TO SEPARATE A 32-BIT INTEGER WORD, V1234, INTO 2 32-BIT INTEGER 
C    WORDS, ONE OF WHICH HAS THE VALUE OF THE HIGHEST 2 BYTES (V12) and
C    ONE OF WHICH HAS THE VALUE OF THE LOWEST 2 BYTES (V34)

      INTEGER v1234,v12,v34,f2
C    f2 is 256**2
      PARAMETER (f2=65536)     

      v12=v1234/f2
      v34=v1234-v12*f2

      RETURN
      END
      
      
      
	SUBROUTINE AHILO(ARA,ISP,IEP,RMX,RMN)
C
C       THIS ROUTINE IS PART OF THE IMX LIKE SPECTRAL ANALYSIS PACKAGE
C       CONVERTED TO RUN ON XXX/PC COMPATIBLE COMPUTERS. IT IS WRITTEN
C       WITH MICROSOFT FORTRAN 77 VERSION 4.0.
C       WRITTEN AND MODIFIED BY
C       KENNETH W. VAN EVERY
C       UC RIVERSIDE, CA.
C
C       AHILO
C       FINDS THE REAL MAX (RMX) AND REAL MIN (RMN) VALUES IN THE RANGE
C       BOUNDED BY ISP AND IEP, IN THE REAL ARRAY ARA.
C
	REAL ARA(1)
	INTEGER  ISP,IEP
	RMX=ARA(ISP)
	RMN=ARA(ISP)
	DO 10 I=ISP,IEP
	IF( ARA(I) .GT. RMX ) RMX=ARA(I)
	IF( ARA(I) .LT. RMN ) RMN=ARA(I)
10      CONTINUE
	RETURN
	END 
	
	
	SUBROUTINE GETCHN(ICHAN,IER)
C
C       THIS ROUTINE IS PART OF THE IMX LIKE SPECTRAL ANALYSIS PACKAGE
C       CONVERTED TO RUN ON XXX/PC COMPATIBLE COMPUTERS. IT IS WRITTEN
C       WITH MICROSOFT FORTRAN 77 VERSION 4.0 AND USES THE MEDIA
C       CYBERNETICS HALO GRAPHICS.
C       WRITTEN AND MODIFIED BY KENNETH W. VAN EVERY
C       UC RIVERSIDE, CA.
C
C       THIS ROUTINE KEEPS TRACK OF A LIMITED NUMBER OF I/O
C       UNIT NUMBERS. ALTHOUGH NEITHER DOS OR MS-F77 SET THIS LIMIT
C       MOST SYSTEMS RUN WITH FILES=20.
C       THIS FEATURE WAS DESIRABLE FOR COMPATIBILITY WITH DG ROUTINES
C       AND FOR ALLOWING MANY MODULAR SUBPROGRAMS TO BE USED TOGETHER
C       WITHOUT WORRYING ABOUT WHO IS USING WHICH UNIT.
C       THIS ROUTINE IS COMPLEMENTED BY PUTCHAN WHICH IS USED TO
C       CLOSE A CHANNEL AND RETURN IS VALUE TO THE USABLE LIST.
C
C       TO USE THIS ROUTINE, CALL IT BEFORE A CHANNEL MUST
C       BE OPENED. IT RETURNS THE NUMBER OF A CHANNEL THAT
C       IS YET UNCLAIMED. THE CALLING ROUTINE CAN CHECK IER
C       TO MAKE SURE THAT NO ERRORS OCCURED. THIS CHANNEL NUMBER
C       MAY THEN BE OPENED AND USED AS NORMAL. WHEN THE PROGRAM
C       IS DONE WITH THE I/O UNIT IT SHOULD BE CLOSED WITH THE PUTCHAN
C       ROUTINE. THIS CLOSES THE CHANNEL AND RETURNS THE UNIT NUMBER TO
C       THE STACK. THIS ROUTINE WILL NOT RETURN THE NUMBERS OF THE
C       PRECONNECTED I/O CHANNELS (0,5,6). THE I/O UNIT=* IS ALWAYS
C       CONNECTED TO THE SCREEN/KEYBOARD COMBINATION.
C
C       THE SYSTEM BREAKS DOWN OF COURSE IF ROUTINES ARE OPENING
C       CHANNELS NOT TRACKED BY GETCHAN OR NOT RETURNING THE NUMBERS
C       TO PUTCHAN FOR CLOSURE.
C
	COMMON /IOCHAN/ IOCH(20) ,ICHN(20),LEFT
	SAVE
C       ASSUME NO ERRORS
	IER=1

	IF( LEFT .EQ. 0 ) THEN
		IER=13
		RETURN
		WRITE(*,*) 'GETCHAN: NO MORE I/O UNITS FREE. '
	ENDIF

	ICHAN=0
	DO 10 I=1,20
	IF(ICHN(I) .NE. 1 )THEN
		ICHAN=IOCH(I)
		LEFT=LEFT-1
		ICHN(I)=1
		RETURN
	ENDIF

10      CONTINUE
	IER =13
	RETURN
	END

	SUBROUTINE PUTCHN(ICHAN,IER)
C
C       THIS ROUTINE IS PART OF THE IMX LIKE SPECTRAL
C       PACKAGE ADAPTED TO RUN ON I?M COMPATIBLE PCS.
C
C       WRITTEN BY    KENNETH W. VAN EVERY
C                     CHEM. DEPT.
C                     U.C. RIVERSIDE, CA.
C
C       THIS ROUTINE IS THE COUNTERPART OF GETCHAN
C       AND IS USED TO CLOSE A CHANNEL WITH THE
C       UNIT NUMBER PREVIOUSLY OBTAINED FROM GETCHAN.
C       SEE THE COMMENTS IN GETCHAN FOR FURTHER DETAILS.
C
	COMMON /IOCHAN/ IOCH(20) ,ICHN(20) ,LEFT
C
C       THE ARRAY IOCH HAS THE CHANNEL NUMBERS WE ARE KEEPING TRACK OF
C       THE ARRAY ICHN CONTAINS ZEROS OR ONES TO INDICATE WHETHER
C       THE CORRESPONDING CHANNEL IN IOCH IS IN USE.
C       LEFT IS USED TO KEEP TRACK OF THE NUMBER OF CHANNELS
C       IN OUR LIST WHICH ARE STILL AVAILABLE.
C
	SAVE
	IER=1
	IF( ICHN(ICHAN) .EQ. 1) THEN
		CLOSE(ICHAN,IOSTAT=IERR)
		ICHN(ICHAN)=0
		IF(LEFT .LT. 20) LEFT=LEFT+1
		IF( IERR .NE. 0) IER=-1
		RETURN
	ENDIF

	WRITE(*,*) 'PUTCHAN:  UNIT ', ICHAN , ' WAS NOT OPEN. '
	RETURN
	END

	BLOCK DATA IOC
	COMMON /IOCHAN/ IOCH(20), ICHN(20) ,LEFT
C       THE CHANNELS WE KEEP TRACK OF.
	DATA IOCH/1,2,3,4,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22/
	DATA LEFT /20/
	END
